package com.example.catchTheLetters.service.impl;

import com.example.catchTheLetters.constant.CommonConstant;
import com.example.catchTheLetters.constant.HttpStatus;
import com.example.catchTheLetters.model.dto.LoginDto;
import com.example.catchTheLetters.model.dto.RegisterDto;
import com.example.catchTheLetters.entity.User;
import com.example.catchTheLetters.model.vo.UserVo;
import com.example.catchTheLetters.service.AuthService;
import com.example.catchTheLetters.service.RedisService;
import com.example.catchTheLetters.utils.JwtUtil;
import com.example.catchTheLetters.utils.R;
import jakarta.annotation.Resource;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.regex.Pattern;

@Service
public class AuthServiceImpl implements AuthService {

    @Resource
    private MongoTemplate mongoTemplate;

    @Resource
    private RedisService redisService;

    @Override
    public R<String> login(LoginDto loginDto) {
        // 根据用户名查询用户信息
        var username = loginDto.getUsername();

        // 判断用户名是否为邮箱
        var patternEmail = Pattern.compile(CommonConstant.EMAIL_REGEX);
        var matcherEmail = patternEmail.matcher(username);

        // 判断用户名是否为手机号
        var patternPhone = Pattern.compile(CommonConstant.PHONE_REGEX);
        var matcherPhone = patternPhone.matcher(username);
        User user;
        if (matcherEmail.matches()) {
            user = mongoTemplate.findOne(new Query(Criteria.where(CommonConstant.EMAIL).is(username).and(CommonConstant.STATUS).ne(0)), User.class);
        } else if (matcherPhone.matches()) {
            user = mongoTemplate.findOne(new Query(Criteria.where(CommonConstant.PHONE).is(username).and(CommonConstant.STATUS).ne(0)), User.class);
        } else {
            user = mongoTemplate.findOne(new Query(Criteria.where(CommonConstant.USERNAME).is(username).and(CommonConstant.STATUS).ne(0)), User.class);
        }
        if (Objects.isNull(user)) {
            return R.fail("用户不存在");
        }
        if (!user.getPassword().equals(loginDto.getPassword())) {
            return R.fail("密码错误");
        }
        Map<String, String> map = new HashMap<>();
        map.put("id", user.getId());
        var token = JwtUtil.getToken(map, CommonConstant.TOKEN_EXPIRE_TIME);
        redisService.hSet(token, user.getId(), CommonConstant.REDIS_EXPIRE_TIME);
        return R.ok(token);
    }

    @Override
    public boolean logout(String token) {
        return redisService.del(token);
    }

    @Override
    public User verify(String token) {
        var map = JwtUtil.getPayload(token);
        String id = null;
        if (map != null) {
            id = map.get("id").asString().replaceAll("\"", "");
        }
        var user = mongoTemplate.findOne(new Query(Criteria.where("id").is(id).and(CommonConstant.STATUS).ne(0)), User.class);
        if (Objects.isNull(user)) {
            return null;
        }
        return user;
    }

    @Override
    public R<UserVo> register(RegisterDto registerDto) {
        var regedUser = getUserByName(registerDto.getUsername());

        // 用户名重复
        if (regedUser != null) {
            R.fail(HttpStatus.USERNAME_CONFLICT.getCode(), HttpStatus.USERNAME_CONFLICT.getDesc());
        }
        var user = new User();
        user.setUsername(registerDto.getUsername());
        user.setPassword(registerDto.getPassword());
        user.setCreateTime(System.currentTimeMillis());
        user.setUpdateTime(System.currentTimeMillis());
        var insert = mongoTemplate.insert(user);
        if (insert.getId() != null) {
            return R.ok(user.toVo());
        } else {
            return R.fail("注册失败,请重试");
        }
    }

    @Override
    public User getUserInfo(String id) {
        return mongoTemplate.findOne(new Query(Criteria.where("id").is(id).and(CommonConstant.STATUS).ne(0)), User.class);
    }

    @Override
    public User getUserByName(String username) {
        return mongoTemplate.findOne(new Query(Criteria.where("username").is(username).and(CommonConstant.STATUS).ne(0)), User.class);
    }

    @Override
    public R<UserVo> update(User user, String token) {
        // 先获取当前账户，比对id是否一致，然后再插入
        var verify = verify(token);
        if (verify == null || !verify.getId().equals(user.getId())) {
            return R.fail(HttpStatus.TOKEN_EXCEPTION.getCode(), HttpStatus.TOKEN_EXCEPTION.getDesc());
        }

        // 如果用户名修改了，检查是否有重复，有重复则返回null，否则修改
        if (!verify.getUsername().equals(user.getUsername())) {
            var regedUser = getUserByName(user.getUsername());
            if (regedUser != null) {
                return R.fail(HttpStatus.USERNAME_CONFLICT.getCode(), HttpStatus.USERNAME_CONFLICT.getDesc());
            }

            verify.setUsername(user.getUsername());
        }

        // 如果密码为空，意味着没修改密码，否则修改密码
        if (user.getPassword() != null && !user.getPassword().isEmpty()) {
            verify.setPassword(user.getPassword());
        }

        verify.setAvatar(user.getAvatar());
        verify.setEmail(user.getEmail());
        verify.setPhone(user.getPhone());
        verify.setSex(user.getSex());
        verify.setIntroduction(user.getIntroduction());
        verify.setUpdateTime(System.currentTimeMillis());

        return R.ok(mongoTemplate.save(verify).toVo());
    }
}
